@@ -20,10 +20,12 @@
 #include "config.h"
 
 #include "gtksourcegutterrenderertext.h"
+#include "gtksourcegutterlines.h"
+#include "gtksourceview-private.h"
 
 /**
  * GtkSourceGutterRendererText:
- * 
+ *
  * Renders text in the gutter.
  *
  * A `GtkSourceGutterRendererText` can be used to render text in a cell of
@@ -38,12 +40,14 @@ typedef struct
 
 typedef struct
 {
-	gchar       *text;
-	PangoLayout *cached_layout;
-	GdkRGBA      cached_color;
-	gsize        text_len;
-	Size         cached_sizes[5];
-	guint        is_markup : 1;
+	gchar          *text;
+	PangoLayout    *cached_layout;
+	PangoAttribute *current_line_bold;
+	PangoAttribute *current_line_color;
+	gsize           text_len;
+	Size            cached_sizes[5];
+	guint           is_markup : 1;
+	guint           has_selection : 1;
 } GtkSourceGutterRendererTextPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutterRendererText, gtk_source_gutter_renderer_text, GTK_SOURCE_TYPE_GUTTER_RENDERER)
@@ -102,12 +106,29 @@ gtk_source_gutter_renderer_text_begin (GtkSourceGutterRenderer *renderer,
 
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
 	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (text);
+	GtkSourceView *view = gtk_source_gutter_renderer_get_view (GTK_SOURCE_GUTTER_RENDERER (renderer));
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	GdkRGBA current;
 
 	GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->begin (renderer, lines);
 
+	priv->has_selection = gtk_text_buffer_get_has_selection (buffer);
+
 	g_clear_object (&priv->cached_layout);
 	priv->cached_layout = gtk_widget_create_pango_layout (GTK_WIDGET (renderer), NULL);
 
+	if (_gtk_source_view_get_current_line_number_color (view, &current))
+	{
+		priv->current_line_color = pango_attr_foreground_new (current.red * 65535,
+		                                                      current.green * 65535,
+		                                                      current.blue * 65535);
+	}
+
+	if (_gtk_source_view_get_current_line_number_bold (view))
+	{
+		priv->current_line_bold = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+	}
+
 	gtk_source_gutter_renderer_text_clear_cached_sizes (text);
 }
 
@@ -120,6 +141,7 @@ gtk_source_gutter_renderer_text_snapshot_line (GtkSourceGutterRenderer *renderer
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
 	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (text);
 	PangoLayout *layout;
+	gboolean clear_attributes;
 	float x;
 	float y;
 	int width;
@@ -131,6 +153,7 @@ gtk_source_gutter_renderer_text_snapshot_line (GtkSourceGutterRenderer *renderer
 	}
 
 	layout = priv->cached_layout;
+	clear_attributes = priv->is_markup;
 
 	if (priv->is_markup)
 	{
@@ -145,6 +168,37 @@ gtk_source_gutter_renderer_text_snapshot_line (GtkSourceGutterRenderer *renderer
 		                       priv->text_len);
 	}
 
+	if (G_UNLIKELY (!priv->has_selection && gtk_source_gutter_lines_is_cursor (lines, line)))
+	{
+		PangoAttrList *attrs = pango_layout_get_attributes (layout);
+
+		if (attrs == NULL)
+		{
+			attrs = pango_attr_list_new ();
+			pango_layout_set_attributes (layout, attrs);
+		}
+		else
+		{
+			pango_attr_list_ref (attrs);
+		}
+
+		if (priv->current_line_color)
+		{
+			pango_attr_list_insert_before (attrs,
+			                               pango_attribute_copy (priv->current_line_color));
+			clear_attributes = TRUE;
+		}
+
+		if (priv->current_line_bold)
+		{
+			pango_attr_list_insert_before (attrs,
+			                               pango_attribute_copy (priv->current_line_bold));
+			clear_attributes = TRUE;
+		}
+
+		pango_attr_list_unref (attrs);
+	}
+
 	gtk_source_gutter_renderer_text_get_size (priv, layout, priv->text_len, &width, &height);
 	gtk_source_gutter_renderer_align_cell (renderer, line, width, height, &x, &y);
 
@@ -154,7 +208,7 @@ gtk_source_gutter_renderer_text_snapshot_line (GtkSourceGutterRenderer *renderer
 	                            ceilf (y),
 	                            layout);
 
-	if (priv->is_markup)
+	if (clear_attributes)
 	{
 		pango_layout_set_attributes (layout, NULL);
 	}
@@ -169,6 +223,8 @@ gtk_source_gutter_renderer_text_end (GtkSourceGutterRenderer *renderer)
 
 	GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->end (renderer);
 
+	g_clear_pointer (&priv->current_line_bold, pango_attribute_destroy);
+	g_clear_pointer (&priv->current_line_color, pango_attribute_destroy);
 	g_clear_object (&priv->cached_layout);
 }
 
