@@ -29,7 +29,7 @@
 
 /**
  * GtkSourceLanguageManager:
- * 
+ *
  * Provides access to [class@Language]s.
  *
  * `GtkSourceLanguageManager` is an object which processes language description
@@ -181,8 +181,8 @@ gtk_source_language_manager_init (GtkSourceLanguageManager *lm)
  * gtk_source_language_manager_new:
  *
  * Creates a new language manager.
- * 
- * If you do not need more than one language manager or a private language manager 
+ *
+ * If you do not need more than one language manager or a private language manager
  * instance then use [func@LanguageManager.get_default] instead.
  *
  * Returns: a new #GtkSourceLanguageManager.
@@ -242,6 +242,10 @@ notify_search_path (GtkSourceLanguageManager *mgr)
  * language files are loaded for the first time. In practice
  * to set a custom search path for a `GtkSourceLanguageManager`,
  * you have to call this function right after creating it.
+ *
+ * Since GtkSourceView 5.4 this function will allow you to provide
+ * paths in the form of "resource:///" URIs to embedded `GResource`s.
+ * They must contain the path of a directory within the `GResource`.
  */
 void
 gtk_source_language_manager_set_search_path (GtkSourceLanguageManager *lm,
@@ -267,6 +271,83 @@ gtk_source_language_manager_set_search_path (GtkSourceLanguageManager *lm,
 	notify_search_path (lm);
 }
 
+/**
+ * gtk_source_language_manager_append_search_path:
+ * @lm: a #GtkSourceLanguageManager.
+ * @path: a directory or a filename.
+ *
+ * Appends @path to the list of directories where the @manager looks for
+ * language files.
+ *
+ * See [method@LanguageManager.set_search_path] for details.
+ *
+ * Since: 5.4
+ */
+void
+gtk_source_language_manager_append_search_path (GtkSourceLanguageManager *lm,
+                                                const gchar              *path)
+{
+	guint len = 0;
+
+	g_return_if_fail (GTK_SOURCE_IS_LANGUAGE_MANAGER (lm));
+	g_return_if_fail (path != NULL);
+
+	if (lm->lang_dirs == NULL)
+		lm->lang_dirs = _gtk_source_utils_get_default_dirs (LANGUAGE_DIR);
+
+	g_return_if_fail (lm->lang_dirs != NULL);
+
+	len = g_strv_length (lm->lang_dirs);
+
+	lm->lang_dirs = g_renew (gchar *,
+	                         lm->lang_dirs,
+	                         len + 2); /* old path + new entry + NULL */
+
+	lm->lang_dirs[len] = g_strdup (path);
+	lm->lang_dirs[len + 1] = NULL;
+
+	notify_search_path (lm);
+}
+
+/**
+ * gtk_source_language_manager_prepend_search_path:
+ * @lm: a #GtkSourceLanguageManager.
+ * @path: a directory or a filename.
+ *
+ * Prepends @path to the list of directories where the @manager looks
+ * for language files.
+ *
+ * See [method@LanguageManager.set_search_path] for details.
+ *
+ * Since: 5.4
+ */
+void
+gtk_source_language_manager_prepend_search_path (GtkSourceLanguageManager *lm,
+                                                 const gchar              *path)
+{
+	guint len = 0;
+	gchar **new_lang_dirs;
+
+	g_return_if_fail (GTK_SOURCE_IS_LANGUAGE_MANAGER (lm));
+	g_return_if_fail (path != NULL);
+
+	if (lm->lang_dirs == NULL)
+		lm->lang_dirs = _gtk_source_utils_get_default_dirs (LANGUAGE_DIR);
+
+	g_return_if_fail (lm->lang_dirs != NULL);
+
+	len = g_strv_length (lm->lang_dirs);
+
+	new_lang_dirs = g_new (gchar *, len + 2);
+	new_lang_dirs[0] = g_strdup (path);
+	memcpy (new_lang_dirs + 1, lm->lang_dirs, (len + 1) * sizeof (gchar*));
+
+	g_free (lm->lang_dirs);
+	lm->lang_dirs = new_lang_dirs;
+
+	notify_search_path (lm);
+}
+
 /**
  * gtk_source_language_manager_get_search_path:
  * @lm: a #GtkSourceLanguageManager.
@@ -608,13 +689,15 @@ pick_lang_for_mime_type (GtkSourceLanguageManager *lm,
  * @content_type: (nullable): a content type (as in GIO API), or %NULL.
  *
  * Picks a [class@Language] for given file name and content type,
- * according to the information in lang files. 
- * 
+ * according to the information in lang files.
+ *
  * Either @filename or @content_type may be %NULL. This function can be used as follows:
  *
  * ```c
  * GtkSourceLanguage *lang;
- * lang = gtk_source_language_manager_guess_language (filename, NULL);
+ * GtkSourceLanguageManager *manager;
+ * lm = gtk_source_language_manager_get_default ();
+ * lang = gtk_source_language_manager_guess_language (manager, filename, NULL);
  * gtk_source_buffer_set_language (buffer, lang);
  * ```
  *
@@ -622,6 +705,7 @@ pick_lang_for_mime_type (GtkSourceLanguageManager *lm,
  *
  * ```c
  * GtkSourceLanguage *lang = NULL;
+ * GtkSourceLanguageManager *manager;
  * gboolean result_uncertain;
  * gchar *content_type;
  *
@@ -632,6 +716,7 @@ pick_lang_for_mime_type (GtkSourceLanguageManager *lm,
  *     content_type = NULL;
  *   }
  *
+ * manager = gtk_source_language_manager_get_default ();
  * lang = gtk_source_language_manager_guess_language (manager, filename, content_type);
  * gtk_source_buffer_set_language (buffer, lang);
  *
