@@ -28,9 +28,6 @@
 #include "gtksourcesnippetchunk-private.h"
 #include "gtksourcesnippetmanager.h"
 #include "gtksourceview-private.h"
-#include "gtksourceutils-private.h"
-
-static void gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets);
 
 static void
 gtk_source_view_snippets_block (GtkSourceViewSnippets *snippets)
@@ -70,36 +67,11 @@ static void
 gtk_source_view_snippets_scroll_to_insert (GtkSourceViewSnippets *snippets)
 {
 	GtkTextMark *mark;
-	GtkTextIter iter;
-	GdkRectangle area;
-	GdkRectangle visible_rect;
-	double x, y;
 
 	g_assert (snippets != NULL);
 
 	mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (snippets->buffer));
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (snippets->buffer), &iter, mark);
-	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (snippets->view), &iter, &area);
-	gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (snippets->view), &visible_rect);
-
-	if (area.x < visible_rect.x)
-		x = area.x;
-	else if (area.x > visible_rect.x + visible_rect.width)
-		x = area.x - visible_rect.width;
-	else
-		x = visible_rect.x;
-
-	if (area.y < visible_rect.y)
-		y = area.y;
-	else if (area.y > visible_rect.y + visible_rect.height)
-		y = area.y - visible_rect.height;
-	else
-		y = visible_rect.y;
-
-	gtk_adjustment_set_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (snippets->view)), x);
-	gtk_adjustment_set_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (snippets->view)), y);
-
-	gtk_source_view_snippets_update_informative (snippets);
+	gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (snippets->view), mark);
 }
 
 static void
@@ -340,17 +312,6 @@ gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets)
 		goto hide_informative;
 	}
 
-	if (snippets->informative == NULL)
-	{
-		snippets->informative = g_object_new (GTK_SOURCE_TYPE_INFORMATIVE,
-		                                      "position", GTK_POS_TOP,
-		                                      "message-type", GTK_MESSAGE_INFO,
-		                                       "icon-name", "completion-snippet-symbolic",
-		                                      NULL);
-		_gtk_source_view_add_assistant (snippets->view,
-		                                GTK_SOURCE_ASSISTANT (snippets->informative));
-	}
-
 	_gtk_source_assistant_set_mark (GTK_SOURCE_ASSISTANT (snippets->informative), chunk->begin_mark);
 	gtk_source_informative_set_message (snippets->informative, tooltip_text);
 
@@ -366,10 +327,7 @@ gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets)
 	return;
 
 hide_informative:
-	if (snippets->informative != NULL)
-	{
-		gtk_widget_hide (GTK_WIDGET (snippets->informative));
-	}
+	gtk_widget_hide (GTK_WIDGET (snippets->informative));
 }
 
 static void
@@ -428,6 +386,14 @@ _gtk_source_view_snippets_init (GtkSourceViewSnippets *snippets,
 		_gtk_source_view_snippets_set_buffer (snippets,
 		                                      GTK_SOURCE_BUFFER (buffer));
 	}
+
+	snippets->informative = g_object_new (GTK_SOURCE_TYPE_INFORMATIVE,
+	                                      "position", GTK_POS_TOP,
+	                                      "message-type", GTK_MESSAGE_INFO,
+	                                      "icon-name", "completion-snippet-symbolic",
+	                                      NULL);
+	_gtk_source_view_add_assistant (snippets->view,
+	                                GTK_SOURCE_ASSISTANT (snippets->informative));
 }
 
 void
@@ -449,11 +415,8 @@ _gtk_source_view_snippets_shutdown (GtkSourceViewSnippets *snippets)
 	g_clear_signal_handler (&snippets->buffer_cursor_moved_handler,
 	                        snippets->buffer);
 
-	if (snippets->informative != NULL)
-	{
-		_gtk_source_view_remove_assistant (snippets->view,
-		                                   GTK_SOURCE_ASSISTANT (snippets->informative));
-	}
+	_gtk_source_view_remove_assistant (snippets->view,
+	                                   GTK_SOURCE_ASSISTANT (snippets->informative));
 
 	if (snippets->snippet_signals != NULL)
 	{
@@ -670,11 +633,6 @@ _gtk_source_view_snippets_pop (GtkSourceViewSnippets *snippets)
 
 	g_return_if_fail (snippets != NULL);
 
-	if (snippets->informative != NULL)
-	{
-		gtk_widget_hide (GTK_WIDGET (snippets->informative));
-	}
-
 	if (snippets->buffer == NULL)
 	{
 		return;
@@ -702,6 +660,8 @@ _gtk_source_view_snippets_pop (GtkSourceViewSnippets *snippets)
 		gtk_source_view_snippets_scroll_to_insert (snippets);
 
 		g_object_unref (snippet);
+
+		gtk_widget_hide (GTK_WIDGET (snippets->informative));
 	}
 
 	snippet = g_queue_peek_head (&snippets->queue);
