@@ -55,7 +55,6 @@ enum {
 	FILTER,
 	FORMAT,
 	READY,
-	SPLIT,
 	N_SIGNALS
 };
 
@@ -213,7 +212,7 @@ constrain_insert_source (gpointer data)
 
 	if (GTK_SOURCE_IS_VIM_VISUAL (current))
 	{
-		gtk_source_vim_visual_warp (GTK_SOURCE_VIM_VISUAL (current), &iter);
+		gtk_source_vim_visual_warp (GTK_SOURCE_VIM_VISUAL (current), &iter, &selection);
 	}
 	else if (!GTK_SOURCE_IS_VIM_INSERT (current) &&
 	         !GTK_SOURCE_IS_VIM_REPLACE (current) &&
@@ -238,7 +237,7 @@ constrain_insert_source (gpointer data)
 		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (buffer), &selection, &selection);
 		visual = gtk_source_vim_visual_new (GTK_SOURCE_VIM_VISUAL_CHAR);
 		gtk_source_vim_state_push (current, visual);
-		gtk_source_vim_visual_warp (GTK_SOURCE_VIM_VISUAL (visual), &iter);
+		gtk_source_vim_visual_warp (GTK_SOURCE_VIM_VISUAL (visual), &iter, &selection);
 		g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMMAND_TEXT]);
 		g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMMAND_BAR_TEXT]);
 	}
@@ -493,29 +492,6 @@ gtk_source_vim_class_init (GtkSourceVimClass *klass)
 		              NULL, NULL,
 		              NULL,
 		              G_TYPE_NONE, 0);
-
-	/**
-	 * GtkSourceVim::split:
-	 * @self: a #GtkSourceVim
-	 * @orientation: a #GtkOrientation for vertical or horizontal
-	 * @new_document: %TRUE if a new document should be created
-	 * @focus_split: %TRUE if the new document should be focused
-	 * @numeric: a numeric value provided with the command such as
-	 *   the number of columns for the split
-	 */
-	signals[SPLIT] =
-		g_signal_new ("split",
-		              G_TYPE_FROM_CLASS (klass),
-		              G_SIGNAL_RUN_LAST,
-		              0,
-		              NULL, NULL,
-		              NULL,
-		              G_TYPE_NONE,
-		              3,
-		              GTK_TYPE_ORIENTATION,
-		              G_TYPE_BOOLEAN,
-		              G_TYPE_BOOLEAN,
-		              G_TYPE_INT);
 }
 
 static void
@@ -564,19 +540,6 @@ gtk_source_vim_get_command_bar_text (GtkSourceVim *self)
 	return "";
 }
 
-void
-gtk_source_vim_emit_split (GtkSourceVim   *self,
-                           GtkOrientation  orientation,
-                           gboolean        new_document,
-                           gboolean        focus_split,
-                           int             numeric)
-{
-	g_return_if_fail (GTK_SOURCE_IS_VIM (self));
-
-	g_signal_emit (self, signals[SPLIT], 0,
-	               orientation, new_document, focus_split, numeric);
-}
-
 void
 gtk_source_vim_reset (GtkSourceVim *self)
 {
