@@ -100,6 +100,7 @@ gtk_source_vim_insert_handle_keypress (GtkSourceVimState *state,
                                        const char        *string)
 {
 	GtkSourceVimInsert *self = (GtkSourceVimInsert *)state;
+	GtkSourceView *view;
 
 	g_assert (GTK_SOURCE_IS_VIM_STATE (state));
 	g_assert (string != NULL);
@@ -112,6 +113,8 @@ gtk_source_vim_insert_handle_keypress (GtkSourceVimState *state,
 		return TRUE;
 	}
 
+	view = gtk_source_vim_state_get_view (state);
+
 	/* Now handle our commands */
 	if ((mods & GDK_CONTROL_MASK) != 0)
 	{
@@ -126,6 +129,11 @@ gtk_source_vim_insert_handle_keypress (GtkSourceVimState *state,
 				gtk_source_vim_state_push (state, gtk_source_vim_insert_literal_new ());
 				return TRUE;
 
+			case GDK_KEY_V:
+				/* For the terminal users out there */
+				g_signal_emit_by_name (view, "paste-clipboard");
+				return TRUE;
+
 			default:
 				break;
 		}
@@ -179,7 +187,6 @@ gtk_source_vim_insert_handle_event (GtkSourceVimState *state,
 	if (gdk_event_get_event_type (event) != GDK_KEY_PRESS)
 		return TRUE;
 
-
 	gtk_source_vim_state_keyval_to_string (keyval, mods, string);
 
 	return GTK_SOURCE_VIM_STATE_GET_CLASS (self)->handle_keypress (state, keyval, keycode, mods, string);
@@ -460,14 +467,13 @@ gtk_source_vim_insert_dispose (GObject *object)
 {
 	GtkSourceVimInsert *self = (GtkSourceVimInsert *)object;
 
-	g_clear_pointer (&self->prefix, g_free);
+	G_OBJECT_CLASS (gtk_source_vim_insert_parent_class)->dispose (object);
 
+	g_clear_pointer (&self->prefix, g_free);
 	gtk_source_vim_state_release (&self->history);
 	gtk_source_vim_state_release (&self->motion);
 	gtk_source_vim_state_release (&self->selection_motion);
 	gtk_source_vim_state_release (&self->text_object);
-
-	G_OBJECT_CLASS (gtk_source_vim_insert_parent_class)->dispose (object);
 }
 
 static void
