@@ -41,12 +41,16 @@ struct _TestWidget
 	GtkCheckButton *draw_spaces_checkbutton;
 	GtkCheckButton *smart_backspace_checkbutton;
 	GtkCheckButton *indent_width_checkbutton;
+	GtkCheckButton *vim_checkbutton;
 	GtkSpinButton *indent_width_spinbutton;
 	GtkLabel *cursor_position_info;
 	GtkSourceStyleSchemeChooserButton *chooser_button;
+	GtkIMContext *vim_im_context;
 	GtkComboBoxText *background_pattern;
 	GtkWidget *top;
 	GtkScrolledWindow *scrolledwindow1;
+	GtkEventController *vim_controller;
+	GtkLabel *command_bar;
 };
 
 struct _TestHoverProvider
@@ -305,7 +309,10 @@ right_margin_position_value_changed_cb (TestWidget    *self,
 					GtkSpinButton *button)
 {
 	gint position = gtk_spin_button_get_value_as_int (button);
+
 	gtk_source_view_set_right_margin_position (self->view, position);
+	gtk_source_view_set_right_margin_position (GTK_SOURCE_VIEW (self->map), position);
+	gtk_widget_queue_resize (GTK_WIDGET (self->map));
 }
 
 static void
@@ -983,6 +990,42 @@ enable_hover_toggled_cb (TestWidget     *self,
 		gtk_source_hover_remove_provider (hover, test_hover_provider);
 }
 
+static void
+vim_checkbutton_toggled_cb (TestWidget     *self,
+                            GtkCheckButton *button)
+{
+	g_assert (TEST_IS_WIDGET (self));
+	g_assert (GTK_IS_CHECK_BUTTON (button));
+
+	if (gtk_check_button_get_active (button))
+	{
+		if (!self->vim_controller)
+		{
+			self->vim_im_context = gtk_source_vim_im_context_new ();
+			gtk_im_context_set_client_widget (self->vim_im_context,
+			                                  GTK_WIDGET (self->view));
+			g_object_bind_property (self->vim_im_context, "command-bar-text", self->command_bar, "label", 0);
+
+			self->vim_controller = gtk_event_controller_key_new ();
+			gtk_event_controller_set_propagation_phase (self->vim_controller, GTK_PHASE_CAPTURE);
+			gtk_event_controller_key_set_im_context (GTK_EVENT_CONTROLLER_KEY (self->vim_controller),
+			                                         GTK_IM_CONTEXT (self->vim_im_context));
+			gtk_widget_add_controller (GTK_WIDGET (self->view),
+			                           self->vim_controller);
+		}
+	}
+	else
+	{
+		if (self->vim_controller)
+		{
+			g_clear_object (&self->vim_im_context);
+			gtk_widget_remove_controller (GTK_WIDGET (self->view),
+			                              self->vim_controller);
+			self->vim_controller = NULL;
+		}
+	}
+}
+
 static void
 test_widget_dispose (GObject *object)
 {
@@ -1023,6 +1066,7 @@ test_widget_class_init (TestWidgetClass *klass)
 	gtk_widget_class_bind_template_callback (widget_class, smart_home_end_changed_cb);
 	gtk_widget_class_bind_template_callback (widget_class, enable_snippets_toggled_cb);
 	gtk_widget_class_bind_template_callback (widget_class, enable_hover_toggled_cb);
+	gtk_widget_class_bind_template_callback (widget_class, vim_checkbutton_toggled_cb);
 
 	gtk_widget_class_bind_template_child (widget_class, TestWidget, view);
 	gtk_widget_class_bind_template_child (widget_class, TestWidget, map);
@@ -1036,8 +1080,10 @@ test_widget_class_init (TestWidgetClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, TestWidget, chooser_button);
 	gtk_widget_class_bind_template_child (widget_class, TestWidget, background_pattern);
 	gtk_widget_class_bind_template_child (widget_class, TestWidget, scrolledwindow1);
+	gtk_widget_class_bind_template_child (widget_class, TestWidget, vim_checkbutton);
+	gtk_widget_class_bind_template_child (widget_class, TestWidget, command_bar);
 
-  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_w, GDK_CONTROL_MASK, "window.close", NULL);
+	gtk_widget_class_add_binding_action (widget_class, GDK_KEY_w, GDK_CONTROL_MASK, "window.close", NULL);
 }
 
 static void
@@ -1134,7 +1180,7 @@ test_widget_init (TestWidget *self)
 	                        "style-scheme",
 	                        self->buffer,
 	                        "style-scheme",
-	                        G_BINDING_SYNC_CREATE);
+	                        G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
 	style_scheme = gtk_source_style_scheme_manager_get_scheme (
 		gtk_source_style_scheme_manager_get_default (),
