@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*- */
 /*
  * This file is part of GtkSourceView
  *
@@ -29,6 +28,8 @@ typedef struct
 	gint newline_type;
 } LoaderTestData;
 
+static GMainLoop *main_loop;
+
 static void
 delete_file (GFile *location)
 {
@@ -76,7 +77,7 @@ load_file_cb (GtkSourceFileLoader *loader,
 	}
 
 	/* finished */
-	gtk_main_quit ();
+	g_main_loop_quit (main_loop);
 }
 
 static void
@@ -93,6 +94,8 @@ test_loader (const gchar *filename,
 	LoaderTestData *data;
 	GError *error = NULL;
 
+	main_loop = g_main_loop_new (NULL, FALSE);
+
 	g_file_set_contents (filename, contents, -1, &error);
 	g_assert_no_error (error);
 
@@ -115,7 +118,7 @@ test_loader (const gchar *filename,
 					   (GAsyncReadyCallback) load_file_cb,
 					   data);
 
-	gtk_main ();
+	g_main_loop_run (main_loop);
 
 	g_slice_free (LoaderTestData, data);
 	delete_file (location);
